import pygame
from OpenGL.GL import *
from OpenGL.GLU import *

from qgl.scene import Group, Transform, Switch, Root, Viewport, Texture, Color, Quad, Sequence, Text, Polyline

        
def render(node):
    glPushMatrix()
    glPushAttrib(GL_CURRENT_BIT|GL_TEXTURE_BIT|GL_ENABLE_BIT|GL_LIGHTING_BIT|GL_POLYGON_BIT)
    if node.__class__ is Group:
        glTranslatef(node.translate[0],node.translate[1],node.translate[2])
        glRotatef(node.angle, node.axis[0], node.axis[1], node.axis[2])
        glScalef(node.scale[0], node.scale[1], node.scale[2])
        for leaf in node.leaves:
            if leaf.__class__ is Texture:
                glEnable(GL_TEXTURE_2D)
                glBindTexture(GL_TEXTURE_2D, leaf.texture.id)
            elif isinstance(leaf, Color):
                glColor4f(leaf.rgba[0], leaf.rgba[1], leaf.rgba[2], leaf.rgba[3])
            elif leaf.__class__ is Polyline:
                glBegin(GL_LINE_STRIP)
                for v in leaf.vertices:
                    glVertex2f(v[0],v[1])
                glEnd()
            elif leaf.__class__ is Quad:
                v = leaf.vertices
                t = leaf.texture_coords
                glBegin(GL_QUADS)
                glNormal3f(0.0,0.0,1.0)
                glTexCoord2f(t[0][0],t[0][1])
                glVertex2f(v[0][0],v[0][1])
                glTexCoord2f(t[1][0],t[1][1])
                glVertex2f(v[1][0],v[1][1])
                glTexCoord2f(t[2][0],t[2][1])
                glVertex2f(v[2][0],v[2][1])
                glTexCoord2f(t[3][0],t[3][1])
                glVertex2f(v[3][0],v[3][1])
                glEnd()
            elif leaf.__class__ is Sequence:
                v = leaf.vertices
                t = leaf.texture.coords[leaf.frame]
                glEnable(GL_TEXTURE_2D)
                glBindTexture(GL_TEXTURE_2D, leaf.texture.id)
                glBegin(GL_QUADS)
                glNormal3f(0.0,0.0,1.0)
                glTexCoord2f(t[0][0], t[0][1])
                glVertex2f(v[0][0],v[0][1])
                glTexCoord2f(t[1][0],t[1][1])
                glVertex2f(v[1][0],v[1][1])
                glTexCoord2f(t[2][0],t[2][1])
                glVertex2f(v[2][0],v[2][1])
                glTexCoord2f(t[3][0],t[3][1])
                glVertex2f(v[3][0],v[3][1])
                glEnd()
            elif leaf.__class__ is Text:
                glCallList(leaf.list)
    elif node.__class__ is Transform:
        glTranslatef(node.translate[0],node.translate[1],node.translate[2])
        glRotatef(node.angle, node.axis[0], node.axis[1], node.axis[2])
        glScalef(node.scale[0], node.scale[1], node.scale[2])
    elif node.__class__ is Viewport:
        glViewport(node.screen_dimensions[0], node.screen_dimensions[1], node.screen_dimensions[2], node.screen_dimensions[3])
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        gluPerspective(45.0, node.aspect, 0.1, 100)
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
    elif node.__class__ is Root:
        glEnable(GL_DEPTH_TEST)
        glDepthFunc(GL_LEQUAL)
        glClearDepth(1.0)
        glFrontFace(GL_CW)
        glShadeModel(GL_SMOOTH)
        glCullFace(GL_BACK)
        glClearColor(node.background_color[0],node.background_color[1],node.background_color[2], node.background_color[3])
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)
    for branch in node.branches:
        render(branch)
    glPopAttrib()
    glPopMatrix()
    
