
import pygame
from pygame.locals import *

import qgl
import qgl.accel
import qgl.interp

def basic_screen(size):
    """
    Returns a root node with viewport and world children.
    """
    root_node = qgl.scene.Root()
    root_node.add(
        qgl.scene.Viewport(
            qgl.scene.World()
        )
    )
    viewport = root_node[0]
    world = viewport[0]
    viewport.screen_dimensions = (0,0,size[0], size[1])
    viewport.aspect = float(size[0]) / size[1]
    return root_node, viewport, world

def main():
    pygame.init()
    flags =  OPENGL|DOUBLEBUF|HWSURFACE
    pygame.display.set_mode((800,600), flags)
    
    #These lines create visitor objects. Visitor objects are accepted by 
    #a node, and traverse the tree performing different operations on the 
    #nodes.
    compiler = qgl.render.Compiler()
    render = qgl.render.Render()
    
    #The compiler visitor traverses the tree, and compiles textures, 
    #sequences etc into OpenGL display lists, texture objects. It must be 
    #used at lease once on the root node, and before any new nodes inserted 
    #into the tree.
    
    #The Render visitor is applied to the root node, and renders objects as
    #it traverses the tree.
    
    clock = pygame.time.Clock()
    
    #These are the basic nodes needed by almost every app.
    #The root node sets up the OpenGL context, the viewport node defined an
    #area of the screen which is ued for rendering, and the world node 
    #provides a sky, ground, foreground, background nodes which nodes can be
    #added to.
    
    root_node, viewport, world = basic_screen((800,600))
    
    #Compile the tree.
    root_node.accept(compiler)
    
    while True:
        for event in pygame.event.get():
            if event.type is QUIT:
                return
            elif event.type is KEYDOWN:
                print clock.get_fps()
                if event.key == 27: 
                    return
        
        #Draw the tree
        root_node.accept(render)
        clock.tick()
        pygame.display.flip()
        
        
main()
